"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemVerCdnCodeResolver = void 0;
const container_definitions_1 = require("@fluidframework/container-definitions");
const isomorphic_fetch_1 = __importDefault(require("isomorphic-fetch"));
const utils_1 = require("./utils");
class FluidPackage {
    constructor(codeDetails, packageUrl) {
        this.codeDetails = codeDetails;
        this.packageUrl = packageUrl;
    }
    async resolve() {
        if (this.resolveP === undefined) {
            this.resolveP = this.resolveCore();
        }
        return this.resolveP;
    }
    async resolveCore() {
        let maybePkg;
        if (typeof this.codeDetails.package === "string") {
            const response = await (0, isomorphic_fetch_1.default)(`${this.packageUrl}/package.json`);
            maybePkg = await response.json();
        }
        else {
            maybePkg = this.codeDetails.package;
        }
        if (!(0, container_definitions_1.isFluidBrowserPackage)(maybePkg)) {
            throw new Error(`Package ${maybePkg === null || maybePkg === void 0 ? void 0 : maybePkg.name} not a Fluid module.`);
        }
        const browser = (0, utils_1.resolveFluidPackageEnvironment)(maybePkg.fluid.browser, this.packageUrl);
        return Object.assign(Object.assign({}, this.codeDetails), { resolvedPackage: Object.assign(Object.assign({}, maybePkg), { fluid: {
                    browser,
                } }), resolvedPackageCacheId: this.packageUrl });
    }
}
/**
 * This code resolver works against cdns that support semantic versioning in the url path of the format:
 * `cdn_base/@package_scope?/package_name@package_version`
 *
 * The `@package_scope?` is optional, and only needed it the package has a scope.
 * The `package_version` can be an npm style semantic version.
 *
 * The `cdn_base` is provided in the config of the Fluid code details, as either a global `config.cdn` property, or
 * a per scope cdn, `config["@package_scope:cdn"]`. A scope specific cdn base will take precedence over
 * the global cdn.
 */
class SemVerCdnCodeResolver {
    constructor() {
        // Cache goes CDN -> package -> entrypoint
        this.fluidPackageCache = new Map();
    }
    async resolveCodeDetails(codeDetails) {
        var _a, _b, _c;
        const parsed = (0, utils_1.extractPackageIdentifierDetails)(codeDetails.package);
        const cdn = (_b = (_a = codeDetails.config) === null || _a === void 0 ? void 0 : _a[`@${parsed.scope}:cdn`]) !== null && _b !== void 0 ? _b : (_c = codeDetails.config) === null || _c === void 0 ? void 0 : _c.cdn;
        const scopePath = parsed.scope !== undefined && parsed.scope.length > 0 ? `@${encodeURI(parsed.scope)}/` : "";
        const packageUrl = parsed.version !== undefined
            ? `${cdn}/${scopePath}${encodeURI(`${parsed.name}@${parsed.version}`)}`
            : `${cdn}/${scopePath}${encodeURI(`${parsed.name}`)}`;
        if (!this.fluidPackageCache.has(packageUrl)) {
            const resolved = new FluidPackage(codeDetails, packageUrl);
            this.fluidPackageCache.set(packageUrl, resolved);
        }
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.fluidPackageCache.get(packageUrl).resolve();
    }
}
exports.SemVerCdnCodeResolver = SemVerCdnCodeResolver;
//# sourceMappingURL=semVerCdnCodeResolver.js.map